program test
#include "petsc/finclude/petscdmplex.h"
#include "petsc/finclude/petscdmlabel.h"
 use petscdmplex
 use petscsys

 implicit none

 DM                                :: dm          ! original dm
 DM                                :: dm_geom     ! cloned dm for metrics
 PetscMPIInt                       :: myid, nproc ! MPI indices
 PetscSF                           :: sf 
 PetscErrorCode                    :: ierr
 character(len=PETSC_MAX_PATH_LEN) :: OutStr
 PetscInt                          :: pst, pend      ! start & end range for points
 PetscInt                          :: vst, vend      ! start & end range for verticies
 PetscInt                          :: est, eend      ! start & end range for edges
 PetscInt                          :: cst, cend      ! start & end range for cells 
 PetscInt                          :: fst, fend      ! start & end range for faces -> not used in 2-D
 PetscSection                      :: s
 type(tVec)                        :: v_root, v_leaf
 PetscReal                         :: const_Real
 PetscReal                         :: const_Real_Conv
 PetscInt                          :: const_Int 

 integer                           :: i

 ! initialize petsc : This section also initializes MPI
 call PetscInitialize(PETSC_NULL_CHARACTER, ierr)
 if(ierr.ne.0) then
   write(*,*) 'Fail to initialize PETSc ...'
   stop
 endif

 ! get MPI size info 
 call MPI_COMM_RANK(MPI_COMM_WORLD, myid, ierr)
 call MPI_COMM_SIZE(MPI_COMM_WORLD, nproc, ierr)
 write(*,*) 'MPI information ... ', myid, nproc

 ! create mesh by default PETSc function
 call DMCreate(PETSC_COMM_WORLD, dm, ierr);CHKERRA(ierr)
 call DMSetType(dm, DMPLEX, ierr);CHKERRA(ierr)
 call DMSetFromOptions(dm, ierr);CHKERRA(ierr)
 !call DMViewFromOptions(dm, PETSC_NULL_VEC, '-dm_view', ierr);CHKERRA(ierr)
 !call DMViewFromOptions(dm, PETSC_NULL_VEC, '-over_dm_view', ierr);CHKERRA(ierr)
 call DMClone(dm, dm_geom, ierr);CHKERRA(ierr)
 call PetscObjectSetName(dm_geom, 'DMPlexTest', ierr);CHKERRA(ierr)

 ! get topological information
 call DMPlexGetChart(dm_geom, pst, pend, ierr);CHKERRA(ierr) ! entire topological points (either height or depth strata)
                                                             ! including everything (i.e., cell, vertices, and interpolated topologies)

 write(OutStr,*) 'Entire topological points range ...', pst, pend, myid, '\n'
 call PetscPrintf(PETSC_COMM_SELF, OutStr, ierr);CHKERRA(ierr) 

 ! using Depth Stratum
 call DMPlexGetDepthStratum(dm_geom, 0, vst, vend, ierr);CHKERRA(ierr) ! verticies
 call DMPlexGetDepthStratum(dm_geom, 1, est, eend, ierr);CHKERRA(ierr) ! edges
 call DMPlexGetDepthStratum(dm_geom, 2, cst, cend, ierr);CHKERRA(ierr) ! cells
 write(OutStr,*) 'From Depth ... cell ', cst, cend, cend-cst, myid, '\n'
 call PetscPrintf(PETSC_COMM_SELF, OutStr, ierr);CHKERRA(ierr) 
 write(OutStr,*) 'From Depth ... edge ', est, eend, eend-est, myid, '\n'
 call PetscPrintf(PETSC_COMM_SELF, OutStr, ierr);CHKERRA(ierr) 
 write(OutStr,*) 'From Depth ... vertex ', vst, vend, vend-vst, myid, '\n'
 call PetscPrintf(PETSC_COMM_SELF, OutStr, ierr);CHKERRA(ierr) 

 ! declare section & star forest with 1-dof on vertex
 call PetscSectionCreate(PETSC_COMM_WORLD, s, ierr);CHKERRA(ierr)
 call PetscSectionSetNumFields(s, 1, ierr);CHKERRA(ierr)
 call PetscSectionSetFieldComponents(s, 0, 1, ierr);CHKERRA(ierr)
 call PetscSectionSetChart(s, vst, vend, ierr);CHKERRA(ierr)
 do i=vst,vend-1
   call PetscSectionSetDof(s, i, 1, ierr);CHKERRA(ierr)
   call PetscSectionSetFieldDof(s, i, 0, 1, ierr);CHKERRA(ierr)
 enddo
 call PetscSectionSetUp(s, ierr);CHKERRA(ierr)
 call DMSetLocalSection(dm_geom, s, ierr);CHKERRA(ierr)
 call PetscSectionDestroy(s, ierr);CHKERRA(ierr)
 call DMGetSectionSF(dm_geom, sf, ierr);CHKERRA(ierr)
 call PetscSFView(sf, PETSC_VIEWER_STDOUT_WORLD, ierr);CHKERRA(ierr)

 ! test on data exchange  
 call DMGetGlobalVector(dm_geom, v_root, ierr);CHKERRA(ierr)
 const_Real = 1.0
 call VecSet(v_root, const_Real, ierr);CHKERRA(ierr)
 call VecViewFromOptions(v_root, PETSC_NULL_VEC, '-vec_view', ierr);CHKERRA(ierr)

 call DMViewFromOptions(dm_geom, PETSC_NULL_VEC, '-dm_view', ierr);CHKERRA(ierr)
 call DMDestroy(dm, ierr);CHKERRA(ierr)
 call DMDestroy(dm_geom, ierr);CHKERRA(ierr)

 ! finalize petsc
 write(OutStr,*) 'done ...', myid, '\n'
 call PetscPrintf(PETSC_COMM_SELF, OutStr, ierr);CHKERRA(ierr) 
 call PetscFinalize(ierr)

 stop
end program test
